from django.db.models.query_utils import Q

from datetime import datetime
import graphene

from aleksis.core.schema.base import FilterOrderList

from ..models import Documentation
from .documentation import (
    DocumentationBatchCreateMutation,
    DocumentationBatchPatchMutation,
    DocumentationCreateMutation,
    DocumentationDeleteMutation,
    DocumentationType,
)


class Query(graphene.ObjectType):
    documentations = FilterOrderList(DocumentationType)
    documentations_by_course_id = FilterOrderList(
        DocumentationType, course_id=graphene.ID(required=True)
    )
    documentations_for_coursebook = FilterOrderList(
        DocumentationType,
        course_id = graphene.ID(required=True),
        date_start=graphene.Date(required=True),
        date_end=graphene.Date(required=True),
    )

    def resolve_documentations_by_course_id(root, info, course_id, **kwargs):
        documentations = Documentation.objects.filter(
            Q(course__pk=course_id) | Q(lesson_event__course__pk=course_id)
        )
        return documentations

    def resolve_documentations_for_coursebook(root, info, course_id, date_start, date_end, **kwargs):
        datetime_start = datetime.combine(date_start, datetime.min.time())
        datetime_end = datetime.combine(date_end, datetime.max.time())

        # TODO: later on, allow getting documentations for other types than courses, e.g. groups or persons
        return Documentation.get_for_coursebook("COURSE", course_id, datetime_start, datetime_end, info.context)


class Mutation(graphene.ObjectType):
    create_documentation = DocumentationCreateMutation.Field()
    create_documentations = DocumentationBatchCreateMutation.Field()
    delete_documentation = DocumentationDeleteMutation.Field()
    update_documentations = DocumentationBatchPatchMutation.Field()
